package com.persagy.apm.energy.report.centermiddleware.service.impl;

import com.persagy.apm.common.response.PoemsFeignResponse;
import com.persagy.apm.energy.report.centermiddleware.client.CenterMiddlewareWebClient;
import com.persagy.apm.energy.report.centermiddleware.model.dto.QueryEnergyDataDTO;
import com.persagy.apm.energy.report.centermiddleware.model.dto.QueryItemInfoDTO;
import com.persagy.apm.energy.report.centermiddleware.model.vo.EnergyDataInfoVO;
import com.persagy.apm.energy.report.centermiddleware.model.vo.EnergyItemInfoVO;
import com.persagy.apm.energy.report.centermiddleware.service.ICenterMiddlewareWebService;
import com.persagy.apm.energy.report.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;

@Service
public class ICenterMiddlewareWebServiceImp implements ICenterMiddlewareWebService {

    @Autowired
    private CenterMiddlewareWebClient centerMiddlewareWebClient;

    @Override
    public Map<String, String> getItemNameMap(QueryItemInfoDTO queryItemInfoDTO) {
        if (null == queryItemInfoDTO) {
            return new HashMap<>();
        }
        PoemsFeignResponse<EnergyItemInfoVO> response = centerMiddlewareWebClient.getItemNameInfo(queryItemInfoDTO);
        List<EnergyItemInfoVO> energyItemInfoVOList = response.getContent();
        if (CollectionUtils.isEmpty(energyItemInfoVOList)) {
            return new HashMap<>();
        }
        Map<String, String> resultMap = new HashMap<>();
        for (EnergyItemInfoVO energyItemInfoVO : energyItemInfoVOList) {
            resultMap.put(energyItemInfoVO.getItemId(), energyItemInfoVO.getItemName());
        }
        return resultMap;
    }

    @Override
    public Map<String, TreeMap<Date, Double>> getItemEnergyDataMap(QueryEnergyDataDTO queryEnergyDataDTO) {
        if (null == queryEnergyDataDTO) {
            return new HashMap<>();
        }
        PoemsFeignResponse<EnergyDataInfoVO> response = centerMiddlewareWebClient.getItemEnergyData(queryEnergyDataDTO);
        List<EnergyDataInfoVO> energyDataInfoVOList = response.getContent();
        if (CollectionUtils.isEmpty(energyDataInfoVOList)) {
            return new HashMap<>();
        }
        Map<String, TreeMap<Date, Double>> resultMap = new HashMap<>();
        try {
            for (EnergyDataInfoVO energyDataInfoVO : energyDataInfoVOList) {
                TreeMap<Date, Double> dateDataMap = resultMap.get(energyDataInfoVO.getItemId());
                if (null == dateDataMap) {
                    dateDataMap = new TreeMap<>();
                }
                dateDataMap.put(DateUtils.str2Date(energyDataInfoVO.getTime(), DateUtils.SDF_SECOND), energyDataInfoVO.getData());
                resultMap.put(energyDataInfoVO.getItemId(), dateDataMap);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return resultMap;
    }
}
