package com.persagy.apm.energy.report.common.utils;

import lombok.extern.slf4j.Slf4j;
import org.nfunk.jep.JEP;
import org.springframework.util.CollectionUtils;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
public class DataUtils {
    /**
     * 对象转Double类型
     *
     * @param object
     * @return
     */
    public static Double parseDouble(Object object) {
        if (object == null) {
            return null;
        }
        if (object instanceof String) {
            return Double.valueOf(object.toString());
        }
        if (object instanceof Integer) {
            return ((Integer) object).doubleValue();
        }
        if (object instanceof Long) {
            return ((Long) object).doubleValue();
        }
        if (object instanceof Double) {
            return (Double) object;
        }
        if (object instanceof BigDecimal) {
            return ((BigDecimal) object).doubleValue();
        }
        return (Double) object;
    }

    /**
     * 按规则进行计算
     *
     * @param standardValue  规则值
     * @param qualifyFormula 规则
     * @param data           要进行规则校验的数据
     * @return
     */
    public static Double getQualifyResult(Double standardValue, String qualifyFormula, Double data) {
        JEP jep = new JEP();
        // 添加常用函数
        jep.addStandardFunctions();
        // 添加常用常量
        jep.addStandardConstants();
        jep.addVariable("standard_value", standardValue);
        jep.addVariable("source", data);
        jep.parseExpression(qualifyFormula);
        return jep.getValue();
    }


    /**
     * 将一组对象中类型为double的属性求和
     *
     * @param objs   要求和的一组对象
     * @param tClass 这组对象的类型
     * @return 一个新的对象，将求和之后的属性赋值给它
     * @author lixing
     * @version V1.0 2021/5/30 10:44 上午
     */
    public static <T> T sumDoubleParams(List<T> objs, Class tClass) {
        try {
            T result = (T) tClass.newInstance();
            if (CollectionUtils.isEmpty(objs)) {
                return result;
            }
            Field[] declaredFields = tClass.getDeclaredFields();
            for (Field declaredField : declaredFields) {
                if (!Double.class.equals(declaredField.getType())) {
                    continue;
                }
                List<T> notNullList = objs.stream().filter(
                        obj -> {
                            try {
                                return declaredField.get(obj) != null;
                            } catch (IllegalAccessException e) {
                                e.printStackTrace();
                                return false;
                            }
                        }).collect(Collectors.toList());
                if (CollectionUtils.isEmpty(notNullList)) {
                    declaredField.set(result, null);
                } else {
                    declaredField.set(result, notNullList.stream().map(
                            obj -> {
                                try {
                                    return new BigDecimal(declaredField.get(obj).toString()) ;
                                } catch (IllegalAccessException e) {
                                    e.printStackTrace();
                                    return new BigDecimal("0");
                                }
                            }).reduce(BigDecimal::add).
                            map(BigDecimal::doubleValue).orElse(0d));
                }

            }
            return result;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 两个double做除法（结果保留4位小数）
     *
     * @param d1 第一个double值
     * @param d2 第二个double值
     * @return 结果
     * @author lixing
     * @version V1.0 2021/6/2 11:02 上午
     */
    public static Double doubleDivide(Double d1, Double d2) {
        // 结果保留的位数
        int scale = 4;
        if (d1 == null || d2 == null) {
            log.error("两个double做除法，其中一个为空或两个都为空, d1: {}, d2: {}", d1, d2);
            return null;
        }
        if (d2.equals(0d)) {
            log.error("double做除法，被除数不能为0");
            return null;
        }
        BigDecimal bigDecimal1 = new BigDecimal(d1);
        BigDecimal bigDecimal2 = new BigDecimal(d2);
        return bigDecimal1.divide(
                bigDecimal2, scale, RoundingMode.HALF_UP).doubleValue();
    }

    /**
     * 两个double做减法
     *
     * @param d1 第一个double值
     * @param d2 第二个double值
     * @return 结果
     * @author lixing
     * @version V1.0 2021/6/2 11:02 上午
     */
    public static Double doubleSubtract(Double d1, Double d2) {
        if (d1 == null || d2 == null) {
            throw new IllegalArgumentException("两个double做减法，其中一个为空或两个都为空");
        }
        BigDecimal bigDecimal1 = new BigDecimal(d1);
        BigDecimal bigDecimal2 = new BigDecimal(d2);
        return bigDecimal1.subtract(bigDecimal2).doubleValue();
    }
}
