package com.persagy.apm.energy.report.monthly.outline.service;

import com.persagy.apm.energy.report.monthly.config.function.model.Function;
import com.persagy.apm.energy.report.monthly.detail.business.model.vo.CostItemVO;
import com.persagy.apm.energy.report.monthly.detail.business.model.vo.CostVO;
import com.persagy.apm.energy.report.monthly.detail.business.model.vo.GroupInfo;
import com.persagy.apm.energy.report.monthly.outline.model.ReportOutline;
import com.persagy.apm.energy.report.saasweb.model.vo.ReportProjectVO;

import java.util.Date;
import java.util.List;

/**
 * 商业报告费用信息service接口
 *
 * @author lixing
 * @version V1.0 2021/5/30 10:26 下午
 **/
public interface IBusinessReportCostInfoService {
    /**
     * 获取费用信息
     *
     * @param reportOutline  报告outline对象
     * @param groupIdList    费用信息点分组列表
     * @param functionIdList 费用信息点列表
     * @return 费用信息
     * @author lixing
     * @version V1.0 2021/5/24 4:14 下午
     */
    CostVO getOpenCost(ReportOutline reportOutline, List<String> groupIdList, List<String> functionIdList);

    /**
     * 获取可同比费用信息
     *
     * @param reportOutline  报告outline对象
     * @param groupIdList    费用信息点分组列表
     * @param functionIdList 费用信息点列表
     * @return 费用信息
     * @author lixing
     * @version V1.0 2021/5/24 4:14 下午
     */
    CostVO getCompareCost(ReportOutline reportOutline, List<String> groupIdList, List<String> functionIdList);

    /**
     * 获取费用条目信息
     *
     * @param functionIdList 费用条目列表
     * @param reportMonth    报告月份
     * @return 费用条目列表
     * @author lixing
     * @version V1.0 2021/5/24 7:15 下午
     */
    List<CostItemVO> getCostItemInfos(List<String> functionIdList, Date reportMonth, List<String> projectIds);

    /**
     * 获取一组项目一个功能点的费用信息
     *
     * @param reportMonth 报告月份
     * @param projectIds  一组项目id
     * @param function    功能点（条目）
     * @return 一组项目一个功能点的费用信息
     * @author lixing
     * @version V1.0 2021/5/30 5:55 下午
     */
    List<CostItemVO> getCostItemsByProjects(Date reportMonth, List<String> projectIds, Function function);

    /**
     * 获取一个条目的费用信息
     *
     * @param reportMonth         报告月份
     * @param reportProjectInfo   报告项目信息
     * @param totalCommercialArea 区域项目总面积
     * @param functionId          功能点id
     * @return 费用对象
     * @author lixing
     * @version V1.0 2021/5/30 5:54 下午
     */
    CostItemVO getCostItemInfo(
            Date reportMonth, ReportProjectVO reportProjectInfo,
            Double totalCommercialArea, String functionId);

    /**
     * 求一组费用条目的统计值
     *
     * @param costItems 一组费用条目
     * @return 一个新的费用条目对象，将统计值属性赋值给它
     * @author lixing
     * @version V1.0 2021/5/30 10:46 上午
     */
    CostItemVO getCostSum(List<CostItemVO> costItems);

    /**
     * 获取费用分组信息
     *
     * @param groupIdList 分组id列表
     * @param reportMonth 报告月份
     * @param projectIds  报告涉及的项目id列表
     * @return 分组对象列表
     * @author lixing
     * @version V1.0 2021/5/24 7:12 下午
     */
    List<GroupInfo<CostItemVO>> getCostGroupInfos(List<String> groupIdList, Date reportMonth, List<String> projectIds);
}
