package com.persagy.apm.energy.report.monthly.outline.service.builder.impl;

import cn.hutool.extra.spring.SpringUtil;
import com.persagy.apm.energy.report.monthly.config.type.constant.enums.BelongTypeEnum;
import com.persagy.apm.energy.report.monthly.config.type.constant.enums.BuildingTypeEnum;
import com.persagy.apm.energy.report.monthly.config.type.model.ReportType;
import com.persagy.apm.energy.report.monthly.config.type.service.IReportTypeService;
import com.persagy.apm.energy.report.monthly.outline.model.ReportOutline;
import com.persagy.apm.energy.report.monthly.outline.service.builder.IBusinessReportBuilder;
import com.persagy.apm.energy.report.monthly.outline.service.builder.IBusinessReportGenerator;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 商业项目报告建造者
 *
 * @author lixing
 * @version V1.0 2021/5/12 2:53 下午
 **/
@Component
@Slf4j
public class BusinessReportGenerator implements IBusinessReportGenerator {
    @Autowired
    private IReportTypeService reportTypeService;

    @Override
    public void generateReport(ReportOutline reportOutline) {
        String reportTypeId = reportOutline.getReportTypeId();
        ReportType reportType = reportTypeService.queryReportTypeDetail(reportTypeId);

        if (BuildingTypeEnum.BUSINESS.getType().equals(reportType.getBuildingType())) {
            IBusinessReportBuilder businessReportBuilder;
            // 如果后续类型增多，可以修改为switch,case或者用map存储映射
            if (BelongTypeEnum.PROJECT.getType().equals(reportType.getBelongType())) {
                businessReportBuilder = SpringUtil.getBean("businessProjectReportBuilder");
            } else {
                businessReportBuilder = SpringUtil.getBean("businessAreaReportBuilder");
            }
            // 生成报告，有事务控制。如果生成成功，更新报告状态为已生成
            boolean generateSuccess = businessReportBuilder.generateNewReportDetail(reportOutline);
            if (!generateSuccess) {
                // 生成失败，回滚上述生成报告操作，更新报告状态为生成失败
                businessReportBuilder.updateReportOutlineInfoWhenFailure(reportOutline);
            }
        }
    }
}
