package com.persagy.apm.energy.report.workorder.service.impl;

import com.persagy.apm.common.response.PoemsFeignResponse;
import com.persagy.apm.energy.report.common.dto.QueryAreaPlatformParamDTO;
import com.persagy.apm.energy.report.common.dto.QueryPjPlatformParamDTO;
import com.persagy.apm.energy.report.common.service.CommonService;
import com.persagy.apm.energy.report.common.utils.DataUtils;
import com.persagy.apm.energy.report.common.utils.DateUtils;
import com.persagy.apm.energy.report.monthly.detail.business.model.Platform;
import com.persagy.apm.energy.report.workorder.client.WorkOrderWebClient;
import com.persagy.apm.energy.report.workorder.model.WorkOrder;
import com.persagy.apm.energy.report.workorder.model.dto.QueryWorkOrderListByParamDTO;
import com.persagy.apm.energy.report.workorder.repository.WorkOrderRepository;
import com.persagy.apm.energy.report.workorder.service.IWorkOrderWebService;
import org.apache.commons.lang.StringUtils;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.elasticsearch.core.query.NativeSearchQuery;
import org.springframework.data.elasticsearch.core.query.NativeSearchQueryBuilder;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.TreeMap;
import java.util.stream.Collectors;


@Service
public class IWorkOrderWebServiceImpl implements IWorkOrderWebService {

    @Resource
    private WorkOrderWebClient workOrderWebClient;
    @Resource
    private CommonService commonService;
    @Autowired
    private WorkOrderRepository workOrderRepository;

    @Override
    public void getPjWorkOrderRate(QueryPjPlatformParamDTO queryPjPlatformParamDTO, Platform platform) {
        TreeMap<Date, Double> date_rateMap = new TreeMap<>();
        Date reportDate = queryPjPlatformParamDTO.getReportDate();
        Date startDate = DateUtils.getFirstDayOfYear(reportDate);
        while (startDate.before(DateUtils.getMonthOff(reportDate, 1))) {
            date_rateMap.put(startDate, getWorkOrderCompleteRate(queryPjPlatformParamDTO.getProjectId(), startDate));
            startDate = DateUtils.getMonthOff(startDate, 1);
        }
        List<Double> monthlySummaries = date_rateMap.values().stream().collect(Collectors.toList());
        platform.setMonthlySummaries(monthlySummaries);
    }

    private Double getWorkOrderCompleteRate(String projectId, Date reportDate, String userId) {
        Double completeRate = null;
        try {
            QueryWorkOrderListByParamDTO paramDTO = new QueryWorkOrderListByParamDTO();
            paramDTO.setUser_id(userId);
            paramDTO.setProject_id(projectId);
            paramDTO.setTimeType("ask_end_time");
            paramDTO.setStart_time(DateUtils.date2Str(reportDate, DateUtils.SDFSECOND));
            paramDTO.setEnd_time(DateUtils.date2Str(DateUtils.getMonthOff(reportDate, 1), DateUtils.SDFSECOND));
            PoemsFeignResponse<Object> response = workOrderWebClient.listWorkOrderByParam(paramDTO);
            Integer dealCount = response.getCount();
            paramDTO.setTimeType("create_time");
            response = workOrderWebClient.listWorkOrderByParam(paramDTO);
            Integer workCount = response.getCount();
            if (null != workCount && workCount != 0) {
                completeRate = Double.valueOf(dealCount) / workCount * 100;
            }
            return completeRate;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return completeRate;
    }


    private Double getWorkOrderCompleteRate(String projectId, Date reportDate) {
        Double completeRate = null;
        try {
            Integer workCount = getWorkOrderNumByParam(projectId, reportDate, "ask_end_time");
            Integer dealCount = getWorkOrderNumByParam(projectId, reportDate, "create_time");
            if (null != workCount && workCount != 0) {
                completeRate = Double.valueOf(dealCount) / workCount * 100;
            }
            return completeRate;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return completeRate;
    }

    private Integer getWorkOrderNumByParam(String projectId, Date reportDate, String timeType) {
        try {
            String startTime = DateUtils.date2Str(reportDate, DateUtils.SDFSECOND);
            String endTime = DateUtils.date2Str(DateUtils.getMonthOff(reportDate, 1), DateUtils.SDFSECOND);
            BoolQueryBuilder queryOrderNumBuilder = createQueryParam(projectId, timeType, startTime, endTime);
            NativeSearchQueryBuilder nativeSearchQueryBuilder = new NativeSearchQueryBuilder();
            nativeSearchQueryBuilder.withQuery(queryOrderNumBuilder);
            NativeSearchQuery query = nativeSearchQueryBuilder.build();
            Page<WorkOrder> resultList = workOrderRepository.search(query);
            if (null != resultList) {
                return resultList.getSize();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    private BoolQueryBuilder createQueryParam(String pjId, String timeType, String startTime, String endTime) {
        BoolQueryBuilder queryBuilder = QueryBuilders.boolQuery();
        queryBuilder.must(QueryBuilders.matchQuery("project_id", pjId));
        //时间查询
        if (StringUtils.isNotBlank(timeType)) {
            if (StringUtils.isNotBlank(startTime)) {
                if ("create_time".equals(timeType)) {
                    queryBuilder.must(QueryBuilders.rangeQuery("create_time").gte(startTime));
                } else if ("ask_end_time".equals(timeType)) {
                    queryBuilder.must(QueryBuilders.rangeQuery("ask_end_time").gte(startTime));
                }
            }
            if (StringUtils.isNotBlank(endTime)) {
                if ("create_time".equals(timeType)) {
                    queryBuilder.must(QueryBuilders.rangeQuery("create_time").lt(endTime));
                } else if ("ask_end_time".equals(timeType)) {
                    queryBuilder.must(QueryBuilders.rangeQuery("ask_end_time").lt(endTime));
                }
            }
        }
        return queryBuilder;
    }


    @Override
    public void getAreaWorkOrderRate(QueryAreaPlatformParamDTO queryAreaPlatformParamDTO, Platform platform) {
        // 合格标准
        Double standardValue = platform.getStandardValue();
        // 校验规则
        String qualifyFormula = platform.getQualifyFormula();
        // 达标数量
        Integer satisfiedCount = 0;
        // 未达标数量
        Integer unsatisfiedCount = 0;
        // 未达标项目id列表
        List<String> unsatisfiedPjIdList = new ArrayList<>();
        for (String pjId : queryAreaPlatformParamDTO.getProjectIdList()) {
            Double completeRate = getWorkOrderCompleteRate(pjId, queryAreaPlatformParamDTO.getReportDate(), queryAreaPlatformParamDTO.getUserId());
            if (null != completeRate && null != standardValue) {
                if (DataUtils.getQualifyResult(standardValue, qualifyFormula, completeRate) == 0.0) {
                    unsatisfiedCount++;
                    unsatisfiedPjIdList.add(pjId);
                } else {
                    satisfiedCount++;
                }
            }
        }
        // 根据未达标项目id列表获取不达标项目名称
        List<String> pjNameList = commonService.getPjNameByPjId(unsatisfiedPjIdList);
        platform.setSatisfiedCount(satisfiedCount);
        platform.setUnsatisfiedCount(unsatisfiedCount);
        platform.setUnsatisfiedProjects(pjNameList);
    }

}
